class_name World extends Node
## Access and flow control for the game world.

@export var pause_enabled := true

@export var spook_manager: SpookManager

@export_category("Game Scenes")
@export var initial_level: PackedScene

@export var pause_scene: PackedScene

@export var kill_screen_scene: PackedScene

var current_level: PackedScene

@onready var level_root: Node3D = %LevelRoot
@onready var ui_root: Control = %UIRoot

static var instance: World


func _ready() -> void:
	World.instance = self
	Game.manager.player_dead.connect(on_player_death)
	load_level(initial_level)


func _unhandled_input(event: InputEvent) -> void:
	if event.is_action_pressed("pause") and pause_enabled:
		pause()


func pause() -> void:
	get_tree().paused = true
	var pause_menu: Control = pause_scene.instantiate()
	ui_root.add_child(pause_menu)
	pause_menu.tree_exiting.connect(unpause)


func unpause() -> void:
	get_tree().paused = false


func load_level(level: PackedScene) -> void:
	for c: Node in level_root.get_children():
		c.queue_free()
	current_level = level
	level_root.add_child(level.instantiate())


func on_player_death() -> void:
	var kill_screen: Control = kill_screen_scene.instantiate()
	kill_screen.tree_exiting.connect(on_game_over)
	ui_root.add_child(kill_screen)


func on_game_over() -> void:
	# TODO: reload from last checkpoint
	# in the mean time, just reload the level
	load_level(current_level)
